// Package httpclient ...
//
// Description : httpclient ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2024-05-31 15:22
package httpclient

import (
	"fmt"
	"net"
	"strings"
	"time"

	"git.zhangdeman.cn/zhangdeman/consts"
	"git.zhangdeman.cn/zhangdeman/network/httpclient/define"
	"git.zhangdeman.cn/zhangdeman/network/httpclient/log"
	"git.zhangdeman.cn/zhangdeman/network/httpclient/validate"
	"git.zhangdeman.cn/zhangdeman/serialize"
	"github.com/go-resty/resty/v2"
	"github.com/tidwall/gjson"
)

// NewHttpClient 获取http client
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:27 2024/5/31
func NewHttpClient(reqConfig *define.Request) (*HttpClient, error) {
	if nil == reqConfig.Logger {
		reqConfig.Logger = log.Get() // 未单独指定日志实例, 则使用全局日志实例
	}
	// 验证配置正确性以及初始化默认值
	if err := validate.RequestConfig(reqConfig); nil != err {
		return nil, err
	}
	if reqConfig.RetryRule == nil {
		reqConfig.RetryRule = &define.RequestRetryRule{
			RetryCount:            0,
			RetryTimeInterval:     0,
			RetryHttpCodeList:     []int64{},
			RetryBusinessCodeList: []string{},
		} // 未指定重试规则, 则使用默认重试规则
	}
	// 初始化成功的 http code list
	if len(reqConfig.SuccessHttpCodeList) == 0 {
		reqConfig.SuccessHttpCodeList = []int{}
	}
	if len(reqConfig.Static) > 0 {
		for loc, valMap := range reqConfig.Static {
			if len(valMap) == 0 {
				continue
			}
			l := strings.ToUpper(loc)
			switch l {
			case consts.RequestDataLocationHeader.String():
				if reqConfig.Header == nil {
					reqConfig.Header = make(map[string]any)
				}
				for k, v := range valMap {
					reqConfig.Header[k] = v
				}
			case consts.RequestDataLocationCookie.String():
				if reqConfig.Cookie == nil {
					reqConfig.Cookie = make(map[string]any)
				}
				for k, v := range valMap {
					reqConfig.Cookie[k] = v
				}
			case consts.RequestDataLocationBody.String():
				if reqConfig.Body == nil {
					reqConfig.Body = make(map[string]any)
				}
				for k, v := range valMap {
					reqConfig.Body[k] = v
				}
			case consts.RequestDataLocationQuery.String():
				if reqConfig.Query == nil {
					reqConfig.Query = make(map[string]any)
				}
				for k, v := range valMap {
					reqConfig.Query[k] = v
				}
			}
		}
	}
	if reqConfig.Header == nil {
		reqConfig.Header = make(map[string]any)
	}
	if ua, exist := reqConfig.Header[consts.HeaderKeyUserAgent.String()]; !exist || nil == ua || fmt.Sprintf("%v", ua) == "" {
		reqConfig.Header[consts.HeaderKeyUserAgent.String()] = "resty-v2@network/httpclient"
	}
	restyClient, restyRequest := NewRestyClient(reqConfig)
	hc := &HttpClient{
		Client:                   restyClient,
		request:                  restyRequest,
		reqConfig:                reqConfig,
		http4xxHandler:           make([]define.Http4xxHandler, 0),
		http5xxHandler:           make([]define.Http5xxHandler, 0),
		httpBusinessErrorHandler: make([]define.HttpBusinessErrorHandler, 0),
		requestFinishHandler:     make([]define.RequestFinishHandler, 0),
	}
	hc.OnRequestFinish(func(req *define.Request, rep *define.Response) {
		if rep.IsSuccess {
			// 请求成功
			log.Record(consts.LogLevelInfo, "接口请求成功", req, rep)
			return
		}
		// 请求失败
		log.Record(consts.LogLevelError, "接口请求失败", req, rep)
	})
	return hc, nil
}

// HttpClient 请求客户端
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:27 2024/5/31
type HttpClient struct {
	*resty.Client
	request                  *resty.Request
	reqConfig                *define.Request
	http4xxHandler           []define.Http4xxHandler
	http5xxHandler           []define.Http5xxHandler
	httpBusinessErrorHandler []define.HttpBusinessErrorHandler
	requestSendErrorHandler  []define.RequestSendErrorHandler
	requestFinishHandler     []define.RequestFinishHandler
}

// OnResponse4xx 4xx处理逻辑
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 18:20 2024/6/1
func (hc *HttpClient) OnResponse4xx(handlerList ...define.Http4xxHandler) {
	hc.http4xxHandler = append(hc.http4xxHandler, handlerList...)
}

// OnResponse5xx 5xx处理逻辑
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 18:21 2024/6/1
func (hc *HttpClient) OnResponse5xx(handlerList ...define.Http5xxHandler) {
	hc.http5xxHandler = append(hc.http5xxHandler, handlerList...)
}

// OnResponseBusinessError 业务错误出路逻辑
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 18:22 2024/6/1
func (hc *HttpClient) OnResponseBusinessError(handlerList ...define.HttpBusinessErrorHandler) {
	hc.httpBusinessErrorHandler = append(hc.httpBusinessErrorHandler, handlerList...)
}

// OnRequestFinish 请求完成事件
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 18:36 2024/6/1
func (hc *HttpClient) OnRequestFinish(handlerList ...define.RequestFinishHandler) {
	hc.requestFinishHandler = append(hc.requestFinishHandler, handlerList...)
}

// getRequestValidateMiddleware 请求验证的Middleware
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:40 2024/5/31
func (hc *HttpClient) getRequestValidateMiddleware() resty.RequestMiddleware {
	return func(client *resty.Client, request *resty.Request) error {
		return nil
	}
}

// getResponseValidateMiddleware 获取响应数据验证的middleware
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:42 2024/5/31
func (hc *HttpClient) getResponseValidateMiddleware() resty.ResponseMiddleware {
	return func(client *resty.Client, response *resty.Response) error {
		return nil
	}
}

// SetRestyClient 设置client
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:54 2024/5/31
func (hc *HttpClient) SetRestyClient(restyClient *resty.Client) {
	hc.Client = restyClient
}

// GetRestyClient 获取 resty client
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:57 2024/5/31
func (hc *HttpClient) GetRestyClient() *resty.Client {
	return hc.Client
}

// Request 发送请求
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 15:52 2024/5/31
func (hc *HttpClient) Request() *define.Response {
	var (
		cacheResult *define.Response
	)

	if cacheResult = hc.getCacheResult(); nil == cacheResult {
		// 未命中缓存, 直接请求后端接口, 无需检测预热等逻辑
		return hc.requestBackendApi()
	}
	// 上面若命中缓存, 则后续缓存实例不可能为nil, 无需判断
	// 判断是否开启预热
	cachePreHeatConfig := hc.reqConfig.CacheInstance.PreHeatConfig()
	if nil == cachePreHeatConfig {
		log.RecordDebug("接口请求命中缓存, PreHeatConfig未返回预热配置, 不做预热处理", map[string]any{}, hc.reqConfig)
		return nil
	}
	log.RecordDebug("接口请求命中缓存, 进行预热策略处理", map[string]any{
		"cache_info": cacheResult.CacheInfo,
	}, hc.reqConfig)
	defer func() {
		// 命中缓存的情况下, 检测缓存预热策略, 判断是否进行缓存预热
		go func() {
			if !cachePreHeatConfig.Enable || (cachePreHeatConfig.MinTTL <= 0 && cachePreHeatConfig.MinPercent <= 0 && !cachePreHeatConfig.Force) {
				// 无预热配置或未启用预热或者未设置预热规则
				log.RecordDebug("接口请求命中缓存, 未配置缓存预热策略", map[string]any{
					"cache_pre_heat_config": cachePreHeatConfig,
				}, hc.reqConfig)
				return
			}
			// 判断是否触发预热
			if cachePreHeatConfig.Force {
				log.RecordDebug("接口请求命中缓存, 强制执行缓存预热, 忽略其他策略配置", map[string]any{
					"cache_pre_heat_config": cachePreHeatConfig,
				}, hc.reqConfig)
				_ = hc.requestBackendApi()
				return
			}
			// 将百分比的配置归一化成最小剩余时间的配置
			if cachePreHeatConfig.MinPercent > 0 {
				expectMinTTL := hc.reqConfig.CacheInstance.CacheTime() * cachePreHeatConfig.MinPercent / 100
				log.RecordDebug("接口请求命中缓存, 配置预热策略：有效时长剩余百分比", map[string]any{
					"cache_pre_heat_config": cachePreHeatConfig,
					"percent_min_ttl":       expectMinTTL,
					"min_ttl":               cachePreHeatConfig.MinTTL,
				}, hc.reqConfig)
				if cachePreHeatConfig.MinTTL == 0 || cachePreHeatConfig.MinTTL > expectMinTTL {
					cachePreHeatConfig.MinTTL = expectMinTTL
				}
			}
			if cachePreHeatConfig.MinTTL <= 0 {
				// 未配置最小剩余时间
				log.RecordDebug("接口请求命中缓存, 未配置预热市场策略, 不执行预热", map[string]any{
					"min_ttl": cachePreHeatConfig.MinTTL,
				}, hc.reqConfig)
				return
			}
			ttl := hc.reqConfig.CacheInstance.TTL(cacheResult.CacheInfo.CacheKey)
			if ttl < 0 {
				// 不存在或者未设置有效期
				log.RecordDebug("接口请求命中缓存, 当前缓存结果不存在或未设置有效期, 不执行预热", map[string]any{
					"min_ttl": cachePreHeatConfig.MinTTL,
					"note":    "预热时间至少在缓存过期前10s触发预热, 以保证足够时间进行预热, 以及不会因为预热尚未完成, 但是大量流量涌入, 进而导致流量穿透",
				}, hc.reqConfig)
				return
			}

			if ttl > cachePreHeatConfig.MinTTL {
				log.RecordDebug("接口请求命中缓存, 缓存结果有效期剩余时长大于配置阈值, 不执行预热", map[string]any{
					"min_ttl":       cachePreHeatConfig.MinTTL,
					"remaining_ttl": ttl,
				}, hc.reqConfig)
				return
			}
			log.RecordDebug("接口请求命中缓存, 缓存结果有效期大于剩余时长小于配置阈值, 触发预热", map[string]any{
				"min_ttl":       cachePreHeatConfig.MinTTL,
				"remaining_ttl": ttl,
			}, hc.reqConfig)
			// 配置了最小剩余时间,并且key剩余有效期小于最小剩余时间
			// 预热加锁, 并发请求触发预热, 仅触发一个即可, 使用接口做key
			if err := hc.reqConfig.CacheInstance.Lock(hc.reqConfig.FullUrl); err != nil {
				log.RecordWarn("接口请求命中缓存, 缓存结果有效期大于剩余时长小于配置阈值, 触发预热, 加锁失败, 未执行预热", map[string]any{
					"min_ttl":       cachePreHeatConfig.MinTTL,
					"remaining_ttl": ttl,
					"err_msg":       err.Error(),
				}, hc.reqConfig)
				return
			}
			log.RecordDebug("接口请求命中缓存, 缓存结果有效期大于剩余时长小于配置阈值, 触发预热, 加锁成功, 执行预热", map[string]any{
				"min_ttl":       cachePreHeatConfig.MinTTL,
				"remaining_ttl": ttl,
				"lock_key":      hc.reqConfig.FullUrl,
			}, hc.reqConfig)
			_ = hc.requestBackendApi()
			if err := hc.reqConfig.CacheInstance.Unlock(hc.reqConfig.FullUrl); nil != err {
				log.RecordError("接口请求命中缓存, 缓存结果有效期大于剩余时长小于配置阈值, 触发预热, 执行预热后, 释放锁失败", map[string]any{
					"min_ttl":       cachePreHeatConfig.MinTTL,
					"remaining_ttl": ttl,
					"lock_key":      hc.reqConfig.FullUrl,
					"err_msg":       err.Error(),
				}, hc.reqConfig)
				return
			}
			log.RecordDebug("接口请求命中缓存, 缓存结果有效期大于剩余时长小于配置阈值, 触发预热, 执行预热后, 释放锁成功", map[string]any{
				"min_ttl":       cachePreHeatConfig.MinTTL,
				"remaining_ttl": ttl,
				"lock_key":      hc.reqConfig.FullUrl,
			}, hc.reqConfig)
		}()
	}()
	// 命中缓存必然请求成功, 直接记录成功日志即可
	log.Record(consts.LogLevelInfo, "接口请求成功:命中缓存", hc.reqConfig, cacheResult)
	return cacheResult
}

// requestBackendApi 请求后端接口
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 18:47 2024/10/9
func (hc *HttpClient) requestBackendApi() *define.Response {
	hc.Client.OnBeforeRequest(hc.getRequestValidateMiddleware())  // 请求参数验证中间件必注册
	hc.Client.OnAfterResponse(hc.getResponseValidateMiddleware()) // 响应验证中间件必注册
	var (
		err error
	)

	response := hc.newResponse()
	for i := 0; i < hc.reqConfig.RetryRule.RetryCount+1; i++ {
		if i > 0 {
			// 非首次请求, 说明是重试, 暂停指定时间间隔
			time.Sleep(time.Duration(hc.reqConfig.RetryRule.RetryTimeInterval) * time.Millisecond)
		}
		if response.Seq > 0 {
			// 说明是重试, 记录上一次的请求信息
			response.RequestFinishTime = time.Now().UnixMilli()
			response.UsedTime = response.RequestFinishTime - response.RequestStartTime
			for _, itemAfterResponse := range hc.requestFinishHandler {
				itemAfterResponse(hc.reqConfig, response)
			}
		}
		response.RequestStartTime = time.Now().UnixMilli() // 每次重置请求时间
		response.RequestFinishTime = 0                     // 清空完成时间
		response.Seq++
		response.RequestCount++
		retryHttpCodeTable := make(map[int64]bool)
		retryBusinessCodeTable := make(map[string]bool)
		if nil != hc.reqConfig.RetryRule {
			for _, httpCode := range hc.reqConfig.RetryRule.RetryHttpCodeList {
				retryHttpCodeTable[httpCode] = true
			}
			for _, businessCode := range hc.reqConfig.RetryRule.RetryBusinessCodeList {
				retryBusinessCodeTable[businessCode] = true
			}
		}
		successHttpCodeTable := make(map[int]bool)
		for _, itemHttpCode := range hc.reqConfig.SuccessHttpCodeList {
			successHttpCodeTable[itemHttpCode] = true
		}
		if response.RestyResponse, err = hc.request.Send(); nil != err {
			errType := define.RequestFailTypeSend
			if err.Error() == resty.ErrRateLimitExceeded.Error() {
				// 命中限流
				errType = define.RequestFailTypeRateLimit
			} else if netErr, ok := err.(net.Error); ok {
				if netErr.Timeout() {
					// 请求超时
					errType = define.RequestFailTypeTimeoutError
				}
			}
			response.FailInfo = &define.ResponseFailInfo{
				Type:    errType,
				Message: err.Error(),
			}
			log.RecordDebug("请求发送出现异常", map[string]any{
				"err_type": errType,
				"err_msg":  err.Error(),
			}, hc.reqConfig)
			if errType == define.RequestFailTypeSend {
				// 命中限流就不重试了
				log.RecordDebug("请求命中限流, 忽略重试策略, 不进行重试", nil, hc.reqConfig)
				break
			}

			if errType == define.RequestFailTypeTimeoutError && !retryHttpCodeTable[499] {
				// 未配置超时重试
				log.RecordDebug("请求超时, 未配置超时重试, 不进行重试", nil, hc.reqConfig)
				break
			}
			log.RecordDebug("请求发送出现异常, 进行重试", map[string]any{
				"err_type":      errType,
				"err_msg":       err.Error(),
				"time_interval": time.Duration(hc.reqConfig.RetryRule.RetryTimeInterval) * time.Millisecond,
			}, hc.reqConfig)
			continue
		}

		if nil == response.RestyResponse {
			response.FailInfo = &define.ResponseFailInfo{
				Type:    define.RequestFailTypeSend,
				Message: "response instance is nil",
			}
			log.RecordDebug("RestyResponse为nil, 准备重试", map[string]any{
				"err_type":      response.FailInfo.Type,
				"time_interval": time.Duration(hc.reqConfig.RetryRule.RetryTimeInterval) * time.Millisecond,
			}, hc.reqConfig)
			continue
		}
		// 解析返回信息
		hc.fillResponseHeader(response)
		hc.fillResponseCookie(response)
		hc.fillResponseBody(response)
		// 配置了当前code为成功, 或者未配置任何code, 当前code为2xx, 则认为请求成功
		isHttpSuccess := successHttpCodeTable[response.HttpCode] || (len(successHttpCodeTable) == 0 && response.HttpCode/100 == 2)
		if !isHttpSuccess {
			// 非 成功
			errType := define.RequestFailTypeServerError
			if response.HttpCode/100 == 4 {
				// 客户端错误
				errType = define.RequestFailTypeClientRequestInvalidError
			}
			response.FailInfo = &define.ResponseFailInfo{
				Type:    errType,
				Message: "http code is " + response.HttpCodeStatus + ", not success",
			}
			log.RecordWarn("请求响应的http状态码非成功", map[string]any{
				"err_type":           errType,
				"err_msg":            response.RestyResponse.Status(),
				"response_http_code": response.HttpCode,
				"success_http_code":  hc.reqConfig.SuccessHttpCodeList,
				"allow_retry":        len(retryHttpCodeTable) == 0 || retryHttpCodeTable[int64(response.HttpCode)],
				"time_interval":      time.Duration(hc.reqConfig.RetryRule.RetryTimeInterval) * time.Millisecond,
			}, hc.reqConfig)
			if len(retryHttpCodeTable) > 0 && !retryHttpCodeTable[int64(response.HttpCode)] {
				// 未配置http code重试
				break
			}
			continue
		}
		if !hc.isCodeSuccess(response) {
			response.FailInfo = &define.ResponseFailInfo{
				Type:    define.RequestFailTypeBusinessError,
				Message: "business code is " + response.Code + ", not success",
			}
			log.RecordWarn("请求响应状态码成功, 业务状态码非成功", map[string]any{
				"err_type":      response.FailInfo.Type,
				"err_msg":       response.Message,
				"response_code": response.Code,
				"success_code":  hc.reqConfig.SuccessCodeList,
				"allow_retry":   len(retryBusinessCodeTable) == 0 || retryBusinessCodeTable[response.Code],
				"time_interval": time.Duration(hc.reqConfig.RetryRule.RetryTimeInterval) * time.Millisecond,
			}, hc.reqConfig)
			if len(retryBusinessCodeTable) > 0 && !retryBusinessCodeTable[response.Code] {
				// 未配置业务code重试
				break
			}
			continue
		}
		response.IsSuccess = true                                                                //设置成功
		response.CacheInfo.SetCache, response.CacheInfo.CacheError = hc.setCacheResult(response) // 设置缓存
		break
	}
	response.RequestFinishTime = time.Now().UnixMilli()
	response.UsedTime = response.RequestFinishTime - response.RequestStartTime
	// 请求完成hook
	for _, itemAfterResponse := range hc.requestFinishHandler {
		itemAfterResponse(hc.reqConfig, response)
	}
	return response
}

// newResponse 默认返回数据
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 17:44 2024/6/1
func (hc *HttpClient) newResponse() *define.Response {
	return &define.Response{
		Header:            map[string]any{},
		Cookie:            map[string]any{},
		Body:              map[string]any{},
		Code:              "",
		Message:           "",
		Data:              "",
		HttpCode:          0,
		HttpCodeStatus:    "",
		ResponseDataRule:  nil,
		Seq:               0,
		RequestStartTime:  0,
		RequestFinishTime: 0,
		UsedTime:          0,
		RestyResponse:     nil,
		IsSuccess:         false,
		CacheInfo: &define.ResponseCacheInfo{
			IsCache:     false,
			SetCache:    false,
			CacheKey:    "",
			CacheValue:  "",
			CacheEnable: nil != hc.reqConfig.CacheInstance && hc.reqConfig.CacheInstance.Enable(),
			CacheError:  nil,
		},
		RequestCount: 0,
		FailInfo:     nil,
	}
}

// fillResponseHeader 填充响应header
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 21:30 2024/6/5
func (hc *HttpClient) fillResponseHeader(response *define.Response) {
	response.Header = map[string]any{}                        // 清空已有数据
	response.HttpCode = response.RestyResponse.StatusCode()   // http状态码
	response.HttpCodeStatus = response.RestyResponse.Status() // http状态码描述
	for headerName, headerValue := range response.RestyResponse.Header() {
		if len(headerValue) > 0 {
			response.Header[headerName] = headerValue[0]
		} else {
			response.Header[headerName] = ""
		}
	}
}

// fillResponseCookie 填充cookie
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 21:32 2024/6/5
func (hc *HttpClient) fillResponseCookie(response *define.Response) {
	response.Cookie = map[string]any{} // 清空已有数据
	for _, cookieValue := range response.RestyResponse.Cookies() {
		response.Cookie[cookieValue.Name] = cookieValue.Value
	}
}

// fillResponseBody 填充响应body
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 21:38 2024/6/5
func (hc *HttpClient) fillResponseBody(response *define.Response) {
	response.Data = string(response.RestyResponse.Body())
	response.Code = gjson.Get(response.Data, hc.reqConfig.CodeField).String()
	response.Message = gjson.Get(response.Data, hc.reqConfig.MessageField).String()
	businessData := gjson.Get(response.Data, hc.reqConfig.DataField)
	if businessData.Value() == nil {
		// data为空指针, 归一化成空对象
		response.Body = map[string]any{}
	} else {
		if businessData.IsArray() {
			// 数组类型的转换
			response.Data = fmt.Sprintf(`{"list":` + businessData.String() + "}")
		} else {
			if businessData.IsObject() {
				// 返回的就是对象
				response.Data = businessData.String()
			} else {
				// 返回是普通类型
				response.Data = serialize.JSON.MarshalForStringIgnoreError(map[string]any{
					"value": businessData.Value(),
				})
			}
		}
		_ = serialize.JSON.UnmarshalWithNumber([]byte(response.Data), &response.Body)
	}

	response.ExtendData = map[string]string{}
	gjson.Parse(response.Data).ForEach(func(key, value gjson.Result) bool {
		if key.String() == hc.reqConfig.CodeField ||
			key.String() == hc.reqConfig.MessageField ||
			key.String() == hc.reqConfig.DataField {
			return true
		}
		response.ExtendData[key.String()] = value.String()
		return true
	})
}

// isHttpCodeSuccess ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 22:48 2024/6/6
func (hc *HttpClient) isCodeSuccess(response *define.Response) bool {
	for _, itemSuccessCode := range hc.reqConfig.SuccessCodeList {
		if itemSuccessCode == response.Code {
			return true
		}
	}
	return false
}

// getCacheResult 获取缓存结果
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 16:04 2024/6/3
func (hc *HttpClient) getCacheResult() *define.Response {
	if nil == hc.reqConfig.CacheInstance {
		log.RecordDebug("接口请求前缓存检测, 未设置缓存实例", map[string]any{}, hc.reqConfig)
		return nil
	}
	if !hc.reqConfig.CacheInstance.Enable() {
		log.RecordDebug("接口请求前缓存检测, 设置缓存实例, 但未启用缓存功能", map[string]any{}, hc.reqConfig)
		return nil
	}
	startTime := time.Now().UnixMilli()
	cacheKey := hc.reqConfig.CacheInstance.GetKey(hc.reqConfig)
	cacheValue := strings.TrimSpace(hc.reqConfig.CacheInstance.GetValue(cacheKey))
	if len(cacheValue) == 0 {
		log.RecordDebug("接口请求前缓存检测, 未读取到缓存数据", map[string]any{}, hc.reqConfig)
		return nil
	}
	response := hc.newResponse()
	if err := serialize.JSON.UnmarshalWithNumber([]byte(cacheValue), response); nil != err {
		log.RecordWarn("接口请求前缓存检测, 读取到缓存数据, 数据解析失败, 将跳过缓存, 请求对应接口", map[string]any{
			"err_msg": err.Error(),
		}, hc.reqConfig)
		return nil
	}
	response.CacheInfo.IsCache = true                                          // 设置缓存标记
	response.RequestStartTime = startTime                                      // 开始时间
	response.RequestFinishTime = time.Now().UnixMilli()                        // 结束时间
	response.UsedTime = response.RequestFinishTime - response.RequestStartTime // 耗时
	response.CacheInfo.CacheKey = cacheKey                                     // 缓存key
	response.CacheInfo.CacheValue = cacheValue                                 // 缓存值
	log.RecordDebug("接口请求前缓存检测, 命中缓存, 直接返回缓存数据", map[string]any{}, hc.reqConfig)
	return response
}

// setCacheResult ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 16:24 2024/6/3
func (hc *HttpClient) setCacheResult(response *define.Response) (bool, error) {
	if nil == response || nil == hc.reqConfig.CacheInstance {
		log.RecordDebug("接口请求成功后, 缓存设置失败", map[string]any{
			"response_is_nil":       response == nil,
			"cache_instance_is_nil": hc.reqConfig.CacheInstance == nil,
		}, hc.reqConfig)
		return false, nil
	}
	// 全局未开启或者当前请求不支持缓存
	globalCacheEnable := hc.reqConfig.CacheInstance.Enable()
	currentRequestAllowCache := hc.reqConfig.CacheInstance.IsAllow(hc.reqConfig, response)
	log.RecordDebug("检测缓存是否允许执行", map[string]any{
		"current_cache_enable": currentRequestAllowCache,
		"global_cache_enable":  globalCacheEnable,
	}, hc.reqConfig)
	if !globalCacheEnable || !currentRequestAllowCache {
		return false, nil
	}
	cacheKey := hc.reqConfig.CacheInstance.GetKey(hc.reqConfig)
	cacheValue := serialize.JSON.MarshalForStringIgnoreError(response)
	if err := hc.reqConfig.CacheInstance.SetValue(cacheKey, cacheValue); nil != err {
		log.RecordWarn("开启结果缓存, 缓存设置失败", map[string]any{
			"current_cache_enable": currentRequestAllowCache,
			"global_cache_enable":  globalCacheEnable,
			"err_msg":              err.Error(),
		}, hc.reqConfig)
		return false, err
	}
	log.RecordDebug("开启结果缓存, 缓存设置成功", map[string]any{
		"current_cache_enable": currentRequestAllowCache,
		"global_cache_enable":  globalCacheEnable,
	}, hc.reqConfig)
	response.CacheInfo.CacheKey = cacheKey
	response.CacheInfo.CacheValue = cacheValue
	return true, nil
}
